-----------------------------------------------------------------------------------------------
-- Client Lua Script for Protogames
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Apollo"
require "Sound"
require "GameLib"
require "PublicEvent"

local DungeonMedalsEnhanced = {}

local kstrNoMedal		= "Protogames:spr_Protogames_Icon_MedalFailed"
local kstrBronzeMedal	= "Protogames:spr_Protogames_Icon_MedalBronze"
local kstrSilverMedal	= "Protogames:spr_Protogames_Icon_MedalSilver"
local kstrGoldMedal		= "Protogames:spr_Protogames_Icon_MedalGold"

function DungeonMedalsEnhanced:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function DungeonMedalsEnhanced:Init()
	Apollo.RegisterAddon(self)
end

function DungeonMedalsEnhanced:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("DungeonMedalsEnhanced.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self)
	
	self:InitializeVars()
end

function DungeonMedalsEnhanced:InitializeVars()
	self.nTimeElapsed = 0
	self.nPointDelta = 0
	self.nPoints	= 0
	self.nBronze = 0
	self.nSilver = 0
	self.nGold = 0
	self.peMatch = nil
	
	-- custom variable
	self.nBasicPoints = 0
	self.nOptionalsPoints = 0
	self.nChallengePoints = 0
	self.nPotentialPoints = 0
	self.nLossPoints = 0
	self.nLossPointsAndPoints	= 0
	self.nBasicPointsBar		= 0
	self.nOptionalsPointsBar	= 0
	self.nChallengePointsBar	= 0
	
	self.bShowDetails = false
end

function DungeonMedalsEnhanced:OnDocumentReady()
	if not self.xmlDoc then
		return
	end
	
	Apollo.RegisterEventHandler("ChangeWorld", 						"Reset", self)
	Apollo.RegisterEventHandler("PublicEventStart",					"CheckForDungeon", self)
	Apollo.RegisterEventHandler("MatchEntered", 					"CheckForDungeon", self)
	Apollo.RegisterEventHandler("PublicEventStatsUpdate", 			"OnPublicEventStatsUpdate", self)
	
	self.timerMatchOneSec = ApolloTimer.Create(1.0, true, "OnOneSecTimer", self)
	self.timerPointsCleanup = ApolloTimer.Create(1.5, true, "OnPointsCleanUpTimer", self)
	self.timerPointsCleanup:Stop()
	
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "DungeonMedalsMain", "FixedHudStratum", self)
	self.wndDetails = Apollo.LoadForm(self.xmlDoc, "DungeonMedalsDetailWindow", "FixedHudStratumLow", self)
	
	if self.wndDetails ~= nil then
		self.wndDetails:Show(false, true)
	end
	
	if not self:CheckForDungeon() then
		self:Reset()
	end
end

function DungeonMedalsEnhanced:Reset()
	self.wndMain:Show(false)
	self.timerMatchOneSec:Stop()
	self.timerPointsCleanup:Stop()
	
	self:InitializeVars()
end

function DungeonMedalsEnhanced:UpdatePoints()
	if self.peMatch then
		self.nPoints					= self.peMatch:GetStat(PublicEvent.PublicEventStatType.MedalPoints)
		self.nLossPointsAndPoints		= (self.nLossPoints * -1) + self.nPoints
		self.nBasicPointsBar			= self.nBasicPoints + self.nPoints
		self.nOptionalsPointsBar		= self.nOptionalsPoints + self.nBasicPointsBar
		self.nChallengePointsBar		= self.nChallengePoints + self.nOptionalsPointsBar
	end
	
	local strVisible	= "ffffffff"
	local strDim		= "66ffffff"
	
	-- Bronze - Tier 1
	local wndBronze = self.wndMain:FindChild("Bronze")
	wndBronze:SetTooltip(Apollo.FormatNumber(self.nBronze, 0, true))
	wndBronze:SetBGColor(self.nPoints >= self.nBronze and strVisible or strDim)
	local wndTier1 = self.wndMain:FindChild("Tier1")
	wndTier1:FindChild("Active"):Show(self.nPoints < self.nBronze)
	-- Challenges
	local wndProgressBar = wndTier1:FindChild("ProgressBarChallenges")
	wndProgressBar:SetMax(self.nBronze)
	wndProgressBar:SetProgress(math.min(self.nBronze, self.nChallengePointsBar))
	-- Optionals
	wndProgressBar = wndTier1:FindChild("ProgressBarOptionals")
	wndProgressBar:SetMax(self.nBronze)
	wndProgressBar:SetProgress(math.min(self.nBronze, self.nOptionalsPointsBar))
	-- Basic
	wndProgressBar = wndTier1:FindChild("ProgressBarBasic")
	wndProgressBar:SetMax(self.nBronze)
	wndProgressBar:SetProgress(math.min(self.nBronze, self.nBasicPointsBar))
	-- Loss
	wndProgressBar = wndTier1:FindChild("ProgressBarLoss")
	wndProgressBar:SetMax(self.nBronze)
	wndProgressBar:SetProgress(math.min(self.nBronze, self.nLossPointsAndPoints))
	-- Normal
	wndProgressBar = wndTier1:FindChild("ProgressBar")
	wndProgressBar:SetMax(self.nBronze)
	wndProgressBar:SetProgress(math.min(self.nBronze, self.nPoints))
	-- Tooltip
	wndProgressBar:SetTooltip("Potential Points: " .. self.nChallengePointsBar .. "\n(Points: " .. self.nPoints .. " Lost Points: " 
			.. self.nLossPoints .. "\n Basic: " .. self.nBasicPoints .. " Optionals: " .. self.nOptionalsPoints .. " Challenges: " .. self.nChallengePoints .. ")")
	
	-- Silver - Tier 2
	local wndSilver = self.wndMain:FindChild("Silver")
	wndSilver:SetTooltip(Apollo.FormatNumber(self.nSilver, 0, true))
	wndSilver:SetBGColor(self.nPoints >= self.nSilver and strVisible or strDim)
	local wndTier2 = self.wndMain:FindChild("Tier2")
	wndTier2:FindChild("Active"):Show(self.nPoints >= self.nBronze and self.nPoints < self.nSilver)
	-- Challenges
	wndProgressBar = wndTier2:FindChild("ProgressBarChallenges")
	wndProgressBar:SetMax(self.nSilver - self.nBronze)
	wndProgressBar:SetProgress(self.nChallengePointsBar > self.nBronze and math.min(self.nSilver, self.nChallengePointsBar - self.nBronze) or 0)
	-- Optionals
	wndProgressBar = wndTier2:FindChild("ProgressBarOptionals")
	wndProgressBar:SetMax(self.nSilver - self.nBronze)
	wndProgressBar:SetProgress(self.nOptionalsPointsBar > self.nBronze and math.min(self.nSilver, self.nOptionalsPointsBar - self.nBronze) or 0)
	-- Basic
	wndProgressBar = wndTier2:FindChild("ProgressBarBasic")
	wndProgressBar:SetMax(self.nSilver - self.nBronze)
	wndProgressBar:SetProgress(self.nBasicPointsBar > self.nBronze and math.min(self.nSilver, self.nBasicPointsBar - self.nBronze) or 0)
	-- Loss
	wndProgressBar = wndTier2:FindChild("ProgressBarLoss")
	wndProgressBar:SetMax(self.nSilver - self.nBronze)
	wndProgressBar:SetProgress(self.nLossPointsAndPoints > self.nBronze and math.min(self.nSilver, self.nLossPointsAndPoints - self.nBronze) or 0)
	-- Normal
	wndProgressBar = wndTier2:FindChild("ProgressBar")
	wndProgressBar:SetMax(self.nSilver - self.nBronze)
	wndProgressBar:SetProgress(self.nPoints > self.nBronze and math.min(self.nSilver, self.nPoints - self.nBronze) or 0)
	--
	wndProgressBar:SetTooltip("Potential Points: " .. self.nChallengePointsBar .. "\n(Points: " .. self.nPoints .. " Lost Points: " 
			.. self.nLossPoints .. "\n Basic: " .. self.nBasicPoints .. " Optionals: " .. self.nOptionalsPoints .. " Challenges: " .. self.nChallengePoints .. ")")
	
	-- Gold - Tier 3
	local wndGold = self.wndMain:FindChild("Gold")
	wndGold:SetTooltip(Apollo.FormatNumber(self.nGold, 0, true))
	wndGold:SetBGColor(self.nPoints >= self.nGold and strVisible or strDim)
	local wndTier3 = self.wndMain:FindChild("Tier3")
	wndTier3:FindChild("Active"):Show(self.nPoints >= self.nSilver and self.nPoints < self.nGold)
	-- Challenges
	wndProgressBar = wndTier3:FindChild("ProgressBarChallenges")
	wndProgressBar:SetMax(self.nGold - self.nSilver)
	wndProgressBar:SetProgress(self.nChallengePointsBar > self.nSilver and math.min(self.nGold, self.nChallengePointsBar - self.nSilver) or 0)
	-- Optionals
	wndProgressBar = wndTier3:FindChild("ProgressBarOptionals")
	wndProgressBar:SetMax(self.nGold - self.nSilver)
	wndProgressBar:SetProgress(self.nOptionalsPointsBar > self.nSilver and math.min(self.nGold, self.nOptionalsPointsBar - self.nSilver) or 0)
	-- Basic
	wndProgressBar = wndTier3:FindChild("ProgressBarBasic")
	wndProgressBar:SetMax(self.nGold - self.nSilver)
	wndProgressBar:SetProgress(self.nBasicPointsBar > self.nSilver and math.min(self.nGold, self.nBasicPointsBar - self.nSilver) or 0)
	-- Loss
	wndProgressBar = wndTier3:FindChild("ProgressBarLoss")
	wndProgressBar:SetMax(self.nGold - self.nSilver)
	wndProgressBar:SetProgress(self.nLossPointsAndPoints > self.nSilver and math.min(self.nGold, self.nLossPointsAndPoints - self.nSilver) or 0)
	-- Normal
	wndProgressBar = wndTier3:FindChild("ProgressBar")
	wndProgressBar:SetMax(self.nGold - self.nSilver)
	wndProgressBar:SetProgress(self.nPoints > self.nSilver and math.min(self.nGold, self.nPoints - self.nSilver) or 0)
	--
	wndProgressBar:SetTooltip("Potential Points: " .. self.nChallengePointsBar .. "\n(Points: " .. self.nPoints .. " Lost Points: " 
			.. self.nLossPoints .. "\n Basic: " .. self.nBasicPoints .. " Optionals: " .. self.nOptionalsPoints .. " Challenges: " .. self.nChallengePoints .. ")")
end

function DungeonMedalsEnhanced:OnPointsCleanUpTimer()
	local nLeft, nTop, nRight, nBottom = self.wndPoints:GetAnchorOffsets()
	local tLoc = WindowLocation.new({ fPoints = { 0.5, 0, 0.5, 0 }, nOffsets = { nLeft-50, nTop-50, nRight-50, nTop-50 }})
	
	self.wndPoints:TransitionMove(tLoc, 1.0)
	self.wndPoints:Show(false, false, 1.0)
	self.nPointDelta = 0
	self.timerPointsCleanup:Stop()
	
end

function DungeonMedalsEnhanced:OnOneSecTimer()
	if self.peMatch then
		self.nTimeElapsed = self.peMatch:GetElapsedTime() > 0 and math.ceil(self.peMatch:GetElapsedTime() / 1000) or self.nTimeElapsed
	else
		self:CheckForDungeon()
		return
	end
	
	local nTime		= self.nTimeElapsed --+3600 (testing hour formatting)
	local nHours		= math.floor(nTime / 3600)
	local nMinutes	= math.floor((nTime - (nHours * 3600)) / 60)
	local nSeconds 	= nTime - (nHours * 3600) - (nMinutes * 60)
	
	local strTime 		= nHours > 0 
		and string.format("%02d:%02d:%02d", nHours, nMinutes, nSeconds) 
		or string.format("%02d:%02d", nMinutes, nSeconds)
	
	self.wndMain:FindChild("Time"):SetText(strTime)
	self.wndMain:FindChild("Points"):SetText(Apollo.FormatNumber(self.nPoints, 0, true))
	
	self.wndMain:Show(self.nTimeElapsed > 0)
end

function DungeonMedalsEnhanced:CheckForDungeon()
	if self.peMatch then
		return true
	end
	
	for key, peCurrent in pairs(PublicEvent.GetActiveEvents()) do
		if peCurrent:ShouldShowMedalsUI() then
			self.nPoints = peCurrent:GetRewardThreshold(PublicEvent.PublicEventRewardTier_None)
			self.nBronze = peCurrent:GetRewardThreshold(PublicEvent.PublicEventRewardTier_Bronze)
			self.nSilver = peCurrent:GetRewardThreshold(PublicEvent.PublicEventRewardTier_Silver)
			self.nGold = peCurrent:GetRewardThreshold(PublicEvent.PublicEventRewardTier_Gold)
				
			self.peMatch = peCurrent
			
			-- update potential points
			self.nChallengePoints = 0
			self.nOptionalsPoints = 0
			self.nBasicPoints = 0
			for key, objective in pairs(peCurrent:GetObjectives()) do

				if objective:GetStatus() == 1 then
			        if objective:GetCategory() == 0 then
			            self.nBasicPoints = self.nBasicPoints + objective:GetMedalPoints()
			        elseif objective:GetCategory() == 1 then
			            self.nOptionalsPoints = self.nOptionalsPoints + objective:GetMedalPoints()
			        elseif objective:GetCategory() == 3 then
			            self.nChallengePoints = self.nChallengePoints + objective:GetMedalPoints()
			        end
				end
			end
	
			-- get Prime difficulty and update UI
			if  GameLib.GetWorldPrimeLevel() > 0 then
				self.wndMain:FindChild("Window2"):SetText("Prime " ..  GameLib.GetWorldPrimeLevel())
			else
			    self.wndMain:FindChild("Window2"):SetText("Normal / Prime")
			end
					
			self.timerMatchOneSec:Start()
			self:UpdatePoints()
			return true
		end
	end
	return false
end

function DungeonMedalsEnhanced:OnPublicEventStatsUpdate(peUpdated)
	if peUpdated:GetEventType() ~= PublicEvent.PublicEventType_Dungeon then
		return
	end
	
	local nCurrentPoints = peUpdated:GetStat(PublicEvent.PublicEventStatType.MedalPoints)
	
	-- update potential points
	self.nChallengePoints = 0
	self.nOptionalsPoints = 0
	self.nBasicPoints = 0
	for key, objective in pairs(peUpdated:GetObjectives()) do
		if objective:GetStatus() == 1 then
	        if objective:GetCategory() == 0 then
	            self.nBasicPoints = self.nBasicPoints + objective:GetMedalPoints()
	        elseif objective:GetCategory() == 1 then
	            self.nOptionalsPoints = self.nOptionalsPoints + objective:GetMedalPoints()
	        elseif objective:GetCategory() == 3 then
	            self.nChallengePoints = self.nChallengePoints + objective:GetMedalPoints()
	        end
		end
	end
	
	if self.nPoints == nCurrentPoints then
		return
	end

	self.timerPointsCleanup:Stop()
	self.timerPointsCleanup:Start()
	
	self.nPointDelta = self.nPointDelta + nCurrentPoints - self.nPoints
	if not self.wndPoints then
		self.wndPoints = Apollo.LoadForm(self.xmlDoc, "DungeonMedalsPlusPoints", "FixedHudStratumLow", self)
	else
		local tWndPointsOffsets = self.wndPoints:GetOriginalLocation():ToTable().nOffsets
		self.wndPoints:SetAnchorOffsets(tWndPointsOffsets[1], tWndPointsOffsets[2], tWndPointsOffsets[3], tWndPointsOffsets[4])
	end
	if self.nPointDelta > 0 then
		ChatSystemLib.PostOnChannel(27, "You have gained " .. self.nPointDelta .. " points.", "[DME]", nil, false)
	else
		ChatSystemLib.PostOnChannel(27, "You have lost " .. self.nPointDelta .. " points.", "[DME]", nil, false)
		self.nLossPoints = self.nLossPoints + self.nPointDelta
	end	
	self.wndPoints:SetData(self.nPointDelta)
	self.wndPoints:SetText("+"..tostring(Apollo.FormatNumber(self.nPointDelta, 0, true)))
	self.wndPoints:Show(true, false, 1.0)
	
	self:UpdatePoints()
end

function DungeonMedalsEnhanced:ShowDetailsWindow()
	self.bShowDetails = not self.bShowDetails
	if self.wndDetails ~= nil then
		self.wndDetails:Show(self.bShowDetails, false)
		self:PopulateDetailsWindow()
	end
end

function DungeonMedalsEnhanced:HideDetailsWindow()
	if self.wndDetails ~= nil then
		self.wndDetails:Show(false, false)
	end
	self.bShowDetails = false
end

function DungeonMedalsEnhanced:PopulateDetailsWindow()
	if self.wndDetails ~= nil then
		textArea = self.wndDetails:FindChild("MedalDetails")
		sDetails = ""
		for key, objective in pairs(self.peMatch:GetObjectives()) do
			if objective:GetStatus() == 1 then
		        if objective:GetCategory() == 0 then
		            sDetails = sDetails .. "Basic: "
		        elseif objective:GetCategory() == 1 then
		            sDetails = sDetails .. "Optional: "
		        elseif objective:GetCategory() == 3 then
		            sDetails = sDetails .. "Challenge: "
		        end
		        sDetails = sDetails .. objective:GetShortDescription() .. " - " .. objective:GetMedalPoints() .. "\n"
			end
		end
		textArea:SetText(sDetails)
	end
end

local DungeonMedalsEnhancedInstance = DungeonMedalsEnhanced:new()
DungeonMedalsEnhancedInstance:Init()